@@ -17,6 +17,8 @@ from .core.utils import (are_keys_equal as _are_keys_equal,
                          to_unique_sorted_items as _to_unique_sorted_items,
                          to_unique_sorted_values as _to_unique_sorted_values)
 from .hints import (Key,
+                    MapFactory,
+                    SetFactory,
                     Value)
 
 
@@ -104,10 +106,10 @@ class Tree(_Tree[Key, Value]):
             else:
                 return parent
 
-    def popmax(self) -> Node:
+    def popmax(self) -> AnyNode:
         node = self.root
         if node is NIL:
-            raise KeyError
+            return node
         elif node.right is NIL:
             self.root = node.left
             return node
@@ -117,10 +119,10 @@ class Tree(_Tree[Key, Value]):
             result, node.right = node.right, node.right.left
             return result
 
-    def popmin(self) -> Node:
+    def popmin(self) -> AnyNode:
         node = self.root
         if node is NIL:
-            raise KeyError
+            return node
         elif node.left is NIL:
             self.root = node.right
             return node
@@ -132,18 +134,17 @@ class Tree(_Tree[Key, Value]):
 
     def predecessor(self, node: Node) -> AnyNode:
         if node.left is NIL:
-            candidate, cursor, key = NIL, self.root, node.key
+            result, cursor, key = NIL, self.root, node.key
             while cursor is not node:
                 if cursor.key < key:
-                    candidate, cursor = cursor, cursor.right
+                    result, cursor = cursor, cursor.right
                 else:
                     cursor = cursor.left
-            return candidate
         else:
             result = node.left
             while result.right is not NIL:
                 result = result.right
-            return result
+        return result
 
     def remove(self, node: Node) -> None:
         parent, key = self.root, node.key
@@ -202,19 +203,18 @@ class Tree(_Tree[Key, Value]):
 
     def successor(self, node: Node) -> AnyNode:
         if node.right is NIL:
-            candidate, cursor, key = NIL, self.root, node.key
+            result, cursor, key = NIL, self.root, node.key
             while cursor is not node:
                 if key < cursor.key:
-                    candidate, cursor = cursor, cursor.left
+                    result, cursor = cursor, cursor.left
                 else:
                     cursor = cursor.right
-            return candidate
         else:
             result = node.right
             while result.left is not NIL:
                 result = result.left
-            return result
+        return result
 
 
-map_ = partial(_map_constructor, Tree.from_components)
-set_ = partial(_set_constructor, Tree.from_components)
+map_ = partial(_map_constructor, Tree.from_components)  # type: MapFactory
+set_ = partial(_set_constructor, Tree.from_components)  # type: SetFactory
