@@ -20,6 +20,8 @@ from .core.utils import (dereference_maybe as _dereference_maybe,
                          to_unique_sorted_items as _to_unique_sorted_items,
                          to_unique_sorted_values as _to_unique_sorted_values)
 from .hints import (Key,
+                    MapFactory,
+                    SetFactory,
                     Value)
 
 
@@ -107,28 +109,26 @@ class Tree(_Tree[Key, Value]):
     @staticmethod
     def predecessor(node: Node) -> Node:
         if node.left is NIL:
-            parent = node.parent
-            while parent is not None and node is parent.left:
-                node, parent = parent, parent.parent
-            return parent
+            result = node.parent
+            while result is not None and node is result.left:
+                node, result = result, result.parent
         else:
             result = node.left
             while result.right is not NIL:
                 result = result.right
-            return result
+        return result
 
     @staticmethod
     def successor(node: Node) -> Node:
         if node.right is NIL:
-            parent = node.parent
-            while parent is not None and node is parent.right:
-                node, parent = parent, parent.parent
-            return parent
+            result = node.parent
+            while result is not None and node is result.right:
+                node, result = result, result.parent
         else:
             result = node.right
             while result.left is not NIL:
                 result = result.left
-            return result
+        return result
 
     @classmethod
     def from_components(cls,
@@ -334,5 +334,5 @@ class Tree(_Tree[Key, Value]):
             parent.right = replacement
 
 
-map_ = partial(_map_constructor, Tree.from_components)
-set_ = partial(_set_constructor, Tree.from_components)
+map_ = partial(_map_constructor, Tree.from_components)  # type: MapFactory
+set_ = partial(_set_constructor, Tree.from_components)  # type: SetFactory
